﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

namespace MyLicenses
{
    public partial class FormNewLicense : Form
    {
        public System.Reflection.AssemblyName assemblyName = null;
        uint LicenseNumber = 0;

        public FormNewLicense()
        {
            InitializeComponent();
            cbSingleMultiple.SelectedIndex = 0;
            txtTolerance.Text = tbTolerance.Value.ToString();
            dtEOS.Enabled = !cbNeverExpires.Checked;
        }

        public FormNewLicense(uint licenseNumber, Guid licenseId) : this()
        {
            LicenseNumber = licenseNumber;
            txtNumber.Text = licenseNumber.ToString();
            txtId.Text = licenseId.ToString();
        }

        private void tbTolerance_ValueChanged(object sender, EventArgs e)
        {
            txtTolerance.Text = tbTolerance.Value.ToString();
        }

        private void cbNeverExpires_CheckedChanged(object sender, EventArgs e)
        {
            dtEOS.Enabled = !cbNeverExpires.Checked;
        }

        private void cbSingleMultiple_SelectedIndexChanged(object sender, EventArgs e)
        {
            pnlMulti.Visible = cbSingleMultiple.SelectedIndex == 1;
            pnlMulti.Enabled = cbSingleMultiple.SelectedIndex == 1;
            pnlSingle.Visible = cbSingleMultiple.SelectedIndex == 0;
            pnlSingle.Enabled = cbSingleMultiple.SelectedIndex == 0;
        }

        private void cbSite_CheckedChanged(object sender, EventArgs e)
        {
            udCount.Enabled = cbDevice.Enabled = cbUser.Enabled = cbConcDevice.Enabled = cbConcUser.Enabled = !cbSite.Checked;
        }

        private void btBrowse_Click(object sender, EventArgs e)
        {
            OpenFileDialog dlg = new OpenFileDialog();
            dlg.Filter = "Assemblies|*.exe;*.dll";
            if (dlg.ShowDialog() != DialogResult.OK) return;
            try
            {
                assemblyName = System.Reflection.AssemblyName.GetAssemblyName(dlg.FileName);
                if (assemblyName == null) return;
                txtAssembly.Text = string.Format("{0} v{1}.{2}", assemblyName.Name, assemblyName.Version.Major, assemblyName.Version.Minor);
                if (cbIgnoreMajor.Checked)
                {
                    txtHashedName.Text = string.Format("{0}", assemblyName.Name);
                }
                else if (cbIgnoreMinor.Checked)
                {
                    txtHashedName.Text = string.Format("{0} v{1}.x", assemblyName.Name, assemblyName.Version.Major);
                }
                else
                {
                    txtHashedName.Text = string.Format("{0} v{1}.{2}", assemblyName.Name, assemblyName.Version.Major, assemblyName.Version.Minor);
                }
                txtName.Text = txtHashedName.Text;
            }
            catch
            {
            }
        }

        private void cbIgnoreMajor_CheckedChanged(object sender, EventArgs e)
        {
            cbIgnoreMinor.Enabled = !cbIgnoreMajor.Checked;
            if (cbIgnoreMajor.Checked)
            {
                txtHashedName.Text = string.Format("{0}", assemblyName.Name);
            }
            else if (cbIgnoreMinor.Checked)
            {
                txtHashedName.Text = string.Format("{0} v{1}.x", assemblyName.Name, assemblyName.Version.Major);
            }
            else
            {
                txtHashedName.Text = string.Format("{0} v{1}.{2}", assemblyName.Name, assemblyName.Version.Major, assemblyName.Version.Minor);
            }
            txtName.Text = txtHashedName.Text;
        }

        private void cbIgnoreMinor_CheckedChanged(object sender, EventArgs e)
        {
            if (cbIgnoreMajor.Checked)
            {
                txtHashedName.Text = string.Format("{0}", assemblyName.Name);
            }
            else if (cbIgnoreMinor.Checked)
            {
                txtHashedName.Text = string.Format("{0} v{1}.x", assemblyName.Name, assemblyName.Version.Major);
            }
            else
            {
                txtHashedName.Text = string.Format("{0} v{1}.{2}", assemblyName.Name, assemblyName.Version.Major, assemblyName.Version.Minor);
            }
            txtName.Text = txtHashedName.Text;
        }

        public string FriendlyName
        {
            get
            {
                return txtName.Text;
            }
        }

        static uint[] crc_table = {
	        0x00000000, 0xEDB88320, 0x36C98560, 0xDB710640, 0x6D930AC0, 0x802B89E0, 0x5B5A8FA0, 0xB6E20C80, 0xDB261580, 0x369E96A0, 0xEDEF90E0, 0x005713C0, 0xB6B51F40, 0x5B0D9C60, 0x807C9A20, 0x6DC41900,
	        0x5BF4A820, 0xB64C2B00, 0x6D3D2D40, 0x8085AE60, 0x3667A2E0, 0xDBDF21C0, 0x00AE2780, 0xED16A4A0, 0x80D2BDA0, 0x6D6A3E80, 0xB61B38C0, 0x5BA3BBE0, 0xED41B760, 0x00F93440, 0xDB883200, 0x3630B120,
	        0xB7E95040, 0x5A51D360, 0x8120D520, 0x6C985600, 0xDA7A5A80, 0x37C2D9A0, 0xECB3DFE0, 0x010B5CC0, 0x6CCF45C0, 0x8177C6E0, 0x5A06C0A0, 0xB7BE4380, 0x015C4F00, 0xECE4CC20, 0x3795CA60, 0xDA2D4940,
	        0xEC1DF860, 0x01A57B40, 0xDAD47D00, 0x376CFE20, 0x818EF2A0, 0x6C367180, 0xB74777C0, 0x5AFFF4E0, 0x373BEDE0, 0xDA836EC0, 0x01F26880, 0xEC4AEBA0, 0x5AA8E720, 0xB7106400, 0x6C616240, 0x81D9E160,
	        0x826A23A0, 0x6FD2A080, 0xB4A3A6C0, 0x591B25E0, 0xEFF92960, 0x0241AA40, 0xD930AC00, 0x34882F20, 0x594C3620, 0xB4F4B500, 0x6F85B340, 0x823D3060, 0x34DF3CE0, 0xD967BFC0, 0x0216B980, 0xEFAE3AA0,
	        0xD99E8B80, 0x342608A0, 0xEF570EE0, 0x02EF8DC0, 0xB40D8140, 0x59B50260, 0x82C40420, 0x6F7C8700, 0x02B89E00, 0xEF001D20, 0x34711B60, 0xD9C99840, 0x6F2B94C0, 0x829317E0, 0x59E211A0, 0xB45A9280,
	        0x358373E0, 0xD83BF0C0, 0x034AF680, 0xEEF275A0, 0x58107920, 0xB5A8FA00, 0x6ED9FC40, 0x83617F60, 0xEEA56660, 0x031DE540, 0xD86CE300, 0x35D46020, 0x83366CA0, 0x6E8EEF80, 0xB5FFE9C0, 0x58476AE0,
	        0x6E77DBC0, 0x83CF58E0, 0x58BE5EA0, 0xB506DD80, 0x03E4D100, 0xEE5C5220, 0x352D5460, 0xD895D740, 0xB551CE40, 0x58E94D60, 0x83984B20, 0x6E20C800, 0xD8C2C480, 0x357A47A0, 0xEE0B41E0, 0x03B3C2C0,
	        0xE96CC460, 0x04D44740, 0xDFA54100, 0x321DC220, 0x84FFCEA0, 0x69474D80, 0xB2364BC0, 0x5F8EC8E0, 0x324AD1E0, 0xDFF252C0, 0x04835480, 0xE93BD7A0, 0x5FD9DB20, 0xB2615800, 0x69105E40, 0x84A8DD60,
	        0xB2986C40, 0x5F20EF60, 0x8451E920, 0x69E96A00, 0xDF0B6680, 0x32B3E5A0, 0xE9C2E3E0, 0x047A60C0, 0x69BE79C0, 0x8406FAE0, 0x5F77FCA0, 0xB2CF7F80, 0x042D7300, 0xE995F020, 0x32E4F660, 0xDF5C7540,
	        0x5E859420, 0xB33D1700, 0x684C1140, 0x85F49260, 0x33169EE0, 0xDEAE1DC0, 0x05DF1B80, 0xE86798A0, 0x85A381A0, 0x681B0280, 0xB36A04C0, 0x5ED287E0, 0xE8308B60, 0x05880840, 0xDEF90E00, 0x33418D20,
	        0x05713C00, 0xE8C9BF20, 0x33B8B960, 0xDE003A40, 0x68E236C0, 0x855AB5E0, 0x5E2BB3A0, 0xB3933080, 0xDE572980, 0x33EFAAA0, 0xE89EACE0, 0x05262FC0, 0xB3C42340, 0x5E7CA060, 0x850DA620, 0x68B52500,
	        0x6B06E7C0, 0x86BE64E0, 0x5DCF62A0, 0xB077E180, 0x0695ED00, 0xEB2D6E20, 0x305C6860, 0xDDE4EB40, 0xB020F240, 0x5D987160, 0x86E97720, 0x6B51F400, 0xDDB3F880, 0x300B7BA0, 0xEB7A7DE0, 0x06C2FEC0,
	        0x30F24FE0, 0xDD4ACCC0, 0x063BCA80, 0xEB8349A0, 0x5D614520, 0xB0D9C600, 0x6BA8C040, 0x86104360, 0xEBD45A60, 0x066CD940, 0xDD1DDF00, 0x30A55C20, 0x864750A0, 0x6BFFD380, 0xB08ED5C0, 0x5D3656E0,
	        0xDCEFB780, 0x315734A0, 0xEA2632E0, 0x079EB1C0, 0xB17CBD40, 0x5CC43E60, 0x87B53820, 0x6A0DBB00, 0x07C9A200, 0xEA712120, 0x31002760, 0xDCB8A440, 0x6A5AA8C0, 0x87E22BE0, 0x5C932DA0, 0xB12BAE80,
	        0x871B1FA0, 0x6AA39C80, 0xB1D29AC0, 0x5C6A19E0, 0xEA881560, 0x07309640, 0xDC419000, 0x31F91320, 0x5C3D0A20, 0xB1858900, 0x6AF48F40, 0x874C0C60, 0x31AE00E0, 0xDC1683C0, 0x07678580, 0xEADF06A0
        };


        static uint crc(string s)
        {
            uint crcsum = 0xFFFFFFFF;

            for (int i = 0; i < s.Length; i++)
            {
                byte c = (byte)s[i];
                c ^= (byte)(crcsum & 0xFF);
                crcsum = crc_table[c] ^ (crcsum >> 8);
            }
            return crcsum;
        }

        public byte[] LicenseInfos
        {
            get
            {
                uint hash = crc(txtHashedName.Text);
                byte[] Infos = new byte[16];
                if (cbIgnoreMinor.Checked) Infos[0] |= 0x20;
                if (cbIgnoreMajor.Checked) Infos[0] |= 0x40;
                if (cbLimitRT.Checked) Infos[0] |= 0x80;
                Utilities.Write(Infos, 1, ((ushort)udPeriod.Value) & 0x7FF);
                Infos[3] = (byte)udFeatures.Value;
                Utilities.Write(Infos, 4, LicenseNumber);
                Utilities.Write(Infos, 12, hash);
                if (!cbNeverExpires.Checked && (dtEOS.Value > MainForm.dtProtector))
                {
                    TimeSpan ts = dtEOS.Value - MainForm.dtProtector;
                    ushort eos = (ushort)ts.TotalDays;
                    Utilities.Write(Infos, 10, eos);
                }
                if (cbSingleMultiple.SelectedIndex == 0)
                {
                    Infos[0] |= (byte)(tbTolerance.Value & 0x1F);
                    if (cbComputer.Checked) Infos[8] |= 1;
                    if (cbUSB.Checked) Infos[8] |= 2;
                    if (cbSMBIOS.Checked) Infos[8] |= 4;
                    if (cbMAC.Checked) Infos[8] |= 8;
                    if (cbUserSingle.Checked) Infos[8] |= 16;
                }
                else
                {
                    Infos[2] |= 0x80;
                    if (cbSite.Checked)
                    {
                        Infos[0] |= 16;
                    }
                    else
                    {
                        if (cbConcUser.Checked) Infos[0] |= 1;
                        if (cbConcDevice.Checked) Infos[0] |= 2;
                        if (cbUser.Checked) Infos[0] |= 4;
                        if (cbDevice.Checked) Infos[0] |= 8;
                        Utilities.Write(Infos, 8, (ushort)udCount.Value);
                    }

                }
                return Infos;
            }
        }
    }
}
