﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

namespace MyLicenses
{
    public partial class FormProperties : Form
    {
        class CommonProps
        {
            [Category("Common")]
            [ReadOnly(true)]
            [DisplayName("License / Subscription Id")]
            public Guid Id { get; set; }
            [Category("Common")]
            [ReadOnly(true)]
            [DisplayName("Software Id")]
            public string SoftwareId { get; set; }
            [Category("Common")]
            [ReadOnly(true)]
            [DisplayName("Friendly Name")]
            public string FriendlyName { get; set; }
            [Category("Common")]
            [ReadOnly(true)]
            public string Version { get; set; }
            [Category("Common")]
            [ReadOnly(true)]
            [DisplayName("Product Key")]
            public string ProductKey { get; set; }
            [Category("Common")]
            [ReadOnly(true)]
            [DisplayName("End Of Subscription")]
            public string EndOfSubscription { get; set; }
            [Category("Common")]
            [ReadOnly(true)]
            [DisplayName("Limit runtime licesenses date to End Of Subscription")]
            public bool Persist { get; set; }
            [Category("Common")]
            [ReadOnly(true)]
            [DisplayName("License / Subscription Number")]
            public uint Number { get; set; }
            [Category("Common")]
            [ReadOnly(true)]
            public byte Features { get; set; }
            [Category("Common")]
            [ReadOnly(true)]
            [DisplayName("Activation Period (days)")]
            public ushort ActivationPeriod { get; set; }
            [Category("Common")]
            [ReadOnly(true)]
            public bool Activated { get; set; }
        }

        class SingleProps : CommonProps
        {
            [Category("Single License")]
            [ReadOnly(true)]
            public byte Tolerance { get; set; }
            [Category("Single License")]
            [ReadOnly(true)]
            [DisplayName("Allowed Kinds")]
            public string Allowedkinds { get; set; }
        }

        class MultipleProps : CommonProps
        {
            [Category("Multiple License")]
            [ReadOnly(true)]
            [DisplayName("License Count")]
            public string LicenseCount { get; set; }
            [Category("Multiple License")]
            [ReadOnly(true)]
            [DisplayName("Allowed Kinds")]
            public string AllowedKinds { get; set; }
            [Category("Multiple License")]
            [ReadOnly(true)]
            [DisplayName("Active Server Id")]
            public string ActiveServerId { get; set; }
            [Category("Multiple License")]
            [ReadOnly(true)]
            [DisplayName("Activated Count")]
            public int ActivatedLicenses { get; set; }
        }

        public FormProperties()
        {
            InitializeComponent();
            props.ViewForeColor = Color.FromArgb(0, 0, 1);
        }

        public FormProperties(DataRow dr) : this()
        {
            this.Text = string.Format("Properties - {0}", dr["Id"]);
            byte[] Infos = ((Guid)dr["Infos"]).ToByteArray();
            CommonProps LicenseProperties;
            bool Multiple = (Infos[2] & 0x80) != 0;
            if (Multiple)
            {
                MultipleProps mp = new MultipleProps();
                LicenseProperties = mp;
                ushort count = Utilities.ReadUInt16(Infos, 8);
                mp.LicenseCount = count == 0 ? "Unlimited (Site)" : count.ToString();
                int kind = Infos[0] & 0x1f;
                List<string> kinds = new List<string>();
                if ((kind & 0x10) != 0) kinds.Add("Site");
                if ((kind & 0x08) != 0) kinds.Add("Device");
                if ((kind & 0x04) != 0) kinds.Add("User");
                if ((kind & 0x02) != 0) kinds.Add("Concurrent Devices");
                if ((kind & 0x01) != 0) kinds.Add("Concurrent Users");
                mp.AllowedKinds = string.Join(" / ", kinds.ToArray());
                mp.ActiveServerId = dr["ActiveServerId"].ToString();
                mp.ActivatedLicenses = (int)dr["nLics"];
            }
            else
            {
                SingleProps sp = new SingleProps();
                LicenseProperties = sp;
                sp.Tolerance = (byte)(Infos[0] & 0x1F);
                ushort kind = Utilities.ReadUInt16(Infos, 8);
                List<string> kinds = new List<string>();
                if ((kind & 0x01) != 0) kinds.Add("Computer");
                if ((kind & 0x02) != 0) kinds.Add("Usb");
                if ((kind & 0x04) != 0) kinds.Add("Smart BIOS");
                if ((kind & 0x08) != 0) kinds.Add("MAC Address");
                if ((kind & 0x10) != 0) kinds.Add("User");
                sp.Allowedkinds = string.Join(" / ", kinds.ToArray());
            }
            int mj = (int)dr["Major"];
            int mn = (int)dr["Minor"];
            string version;
            if (mj == -1) version = "x.y";
            else if (mn == -1) version = string.Format("{0}.x", mj);
            else version = string.Format("{0}.{1}", mj, mn);
            LicenseProperties.Version = version;
            LicenseProperties.Id = (Guid)dr["Id"];
            LicenseProperties.SoftwareId = string.Format("0x{0}", (unchecked((ulong)(long)dr["SoftwareId"])).ToString("X").PadLeft(16, '0'));
            LicenseProperties.FriendlyName = dr["SoftwareName"].ToString();
            LicenseProperties.ProductKey = dr["ProductKey"].ToString();
            LicenseProperties.Persist = (Infos[0] & 0x80) != 0;
            uint ts = Utilities.ReadUInt16(Infos, 10);
            if (ts != 0)
            {
                DateTime eos = MainForm.dtProtector.AddDays(ts);
                LicenseProperties.EndOfSubscription = eos.ToString();
            }
            else
            {
                LicenseProperties.EndOfSubscription = "Permanent License";
            }
            LicenseProperties.Number = Utilities.ReadUInt32(Infos, 4);
            LicenseProperties.Features=Infos[3];
            LicenseProperties.ActivationPeriod = (ushort)(Utilities.ReadUInt16(Infos, 1) & 0x7FF);
            LicenseProperties.Activated = (int)dr["nLics"] != 0;
            props.SelectedObject = LicenseProperties;
        }
    }
}
