﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

using PvLogiciels.dotNetProtector.CloudLicensing;

namespace MyLicenses
{
    public partial class MainForm : Form
    {
        internal static DateTime dtProtector = new DateTime(2016, 1, 1);

        class ActivationKindGridViewTextBoxCell : DataGridViewTextBoxCell
        {
            static string[] ActivationKinds = { "Computer", "USB", "SMBIOS", "MAC", "User" };
            protected override object GetFormattedValue(object value, int rowIndex, ref DataGridViewCellStyle cellStyle, TypeConverter valueTypeConverter, TypeConverter formattedValueTypeConverter, DataGridViewDataErrorContexts context)
            {
                try
                {
                    int kind=Convert.ToInt32(value);
                    if(kind==255) return "Instant";
                    return ActivationKinds[kind];
                }
                catch
                {
                    return base.GetFormattedValue(value, rowIndex, ref cellStyle, valueTypeConverter, formattedValueTypeConverter, context);
                }
            }
        }

        class Hex64GridViewTextBoxCell : DataGridViewTextBoxCell
        {
            protected override object GetFormattedValue(object value, int rowIndex, ref DataGridViewCellStyle cellStyle, TypeConverter valueTypeConverter, TypeConverter formattedValueTypeConverter, DataGridViewDataErrorContexts context)
            {
                try
                {
                    ulong ulval = unchecked((ulong)((long)value));
                    return ulval.ToString("X").PadLeft(16, '0');
                }
                catch
                {
                    return base.GetFormattedValue(value, rowIndex, ref cellStyle, valueTypeConverter, formattedValueTypeConverter, context);
                }
            }
        }

        byte[] PublisherKey;
        byte[] Keyset;

        CloudLicensingClient clt = null;
        BindingSource licenseBindingSource;
        BindingSource activationBindingSource;

        Guid currentActivationId = Guid.Empty;
        Guid currentLicenseId = Guid.Empty;
        DataRow currentLicenseRow = null;
        DataGridViewRow currentLicenseViewRow = null;
        DataRelation drLicenseActivations = null;

        public MainForm()
        {
            InitializeComponent();
            Keyset = LicenseStudio.Keyset;
            PublisherKey = CloudLicensingClient.ExtractPublisherKey(Keyset);
            if (PublisherKey == null)
            {
                MessageBox.Show("Error: Keyset not extended", "License Studio");
            }
            else
            {

                clt = new CloudLicensingClient("http://activation.pvlog.com/dotNetProtectorActivationServer", PublisherKey);
                Reload();
            }
        }

        void Reload()
        {
            if (clt == null) return;    // keyset not extended
            licenseBindingSource = new BindingSource();
            activationBindingSource = new BindingSource();
            DataSet ds = clt.GetMyLicenses();
            if (ds == null)
            {
                MessageBox.Show(this, "Communication Error, try to refresh later", "License Studio");
                return;
            }
            object o = ds.ExtendedProperties["CloudLicensingError"];
            if (o != null)
            {
                byte b = (byte)o;
                if (b == 0xFF)
                {
                    MessageBox.Show(this, "Authentication Error !", "License Studio");
                }
                else
                {
                    MessageBox.Show(this, string.Format("Communication Error {0}, try to refresh later", b), "License Studio");
                }
                return;
            }
            drLicenseActivations = new DataRelation("LicenseActivations", ds.Tables["Licenses"].Columns["Id"], ds.Tables["Activation"].Columns["LicenseId"]);
            ds.Relations.Add(drLicenseActivations);
            DataTable dtLicenses = ds.Tables["Licenses"];
            DataColumn dcEos = dtLicenses.Columns.Add("EndOfSubscription", typeof(string));
            foreach (DataRow dr in dtLicenses.Rows)
            {
                Guid guidInfos = (Guid)dr["Infos"];
                byte[] Infos = guidInfos.ToByteArray();
                uint ts = Utilities.ReadUInt16(Infos, 10);
                if (ts == 0) continue;
                DateTime eos = dtProtector.AddDays(ts);
                dr["EndOfSubscription"] = eos.ToString();
            }
            dtLicenses.Columns["VersionMajor"].ColumnName = "Major";
            dtLicenses.Columns["VersionMinor"].ColumnName = "Minor";
            licenseBindingSource.DataSource = ds;
            licenseBindingSource.DataMember = "Licenses";
            activationBindingSource.DataSource = licenseBindingSource;
            activationBindingSource.DataMember = "LicenseActivations";
            dvLicenses.DataSource = licenseBindingSource;
            dvActivations.DataSource = activationBindingSource;
            dvActivations.Columns["ActivationKind"].CellTemplate = new ActivationKindGridViewTextBoxCell();
            dvActivations.Columns["HwHash"].CellTemplate = new Hex64GridViewTextBoxCell();
            dvActivations.Columns["HwIdentity"].CellTemplate = new Hex64GridViewTextBoxCell();
            dvLicenses.Columns["SoftwareId"].CellTemplate = new Hex64GridViewTextBoxCell();
            dvLicenses.Columns["Infos"].Visible = false;
            dvLicenses.Columns["ActiveServerId"].Visible = false;
            dvActivations.Columns["LicenseId"].Visible = false;
        }

        private void activationsContextMenu_Closed(object sender, ToolStripDropDownClosedEventArgs e)
        {
            dvActivations.ContextMenuStrip = gridviewContextMenuStrip;
        }

        private void dvActivations_RowContextMenuStripNeeded(object sender, DataGridViewRowContextMenuStripNeededEventArgs e)
        {
            if (e.RowIndex < 0 || e.RowIndex >= dvActivations.Rows.Count) return;
            dvActivations.ContextMenuStrip = activationsContextMenu;
            foreach (DataGridViewCell c in dvActivations.SelectedCells) c.Selected = false;
            dvActivations.Rows[e.RowIndex].Selected = true;
            currentActivationId = (Guid)dvActivations.Rows[e.RowIndex].Cells["Id"].Value;
        }

        private void subscriptionsContextMenu_Closed(object sender, ToolStripDropDownClosedEventArgs e)
        {
            dvLicenses.ContextMenuStrip = gridviewContextMenuStrip;
        }

        private void licensesContextMenu_Closed(object sender, ToolStripDropDownClosedEventArgs e)
        {
            dvLicenses.ContextMenuStrip = gridviewContextMenuStrip;
        }

        private void dvLicenses_RowContextMenuStripNeeded(object sender, DataGridViewRowContextMenuStripNeededEventArgs e)
        {
            if (e.RowIndex < 0 || e.RowIndex >= dvLicenses.Rows.Count) return;
            foreach (DataGridViewCell c in dvLicenses.SelectedCells) c.Selected = false;
            currentLicenseViewRow = dvLicenses.Rows[e.RowIndex];
            currentLicenseViewRow.Selected = true;
            DataRowView dv = dvLicenses.Rows[e.RowIndex].DataBoundItem as DataRowView;
            if (dv == null) currentLicenseRow = null; else currentLicenseRow = dv.Row;
            currentLicenseId = (Guid)dvLicenses.Rows[e.RowIndex].Cells["Id"].Value;
            string eos = dvLicenses.Rows[e.RowIndex].Cells["EndOfSubscription"].Value as string;
            dvLicenses.ContextMenuStrip = string.IsNullOrEmpty(eos) ? licensesContextMenu : subscriptionsContextMenu;
        }

        private void deActivateToolStripMenuItem_Click(object sender, EventArgs e)
        {
            clt.ReleaseActivation(currentActivationId);
            Reload();
        }

        private void renewToolStripMenuItem_Click(object sender, EventArgs e)
        {
            FormRenew frmRenew = new FormRenew(currentLicenseRow);
            if (frmRenew.ShowDialog(this) != DialogResult.OK) return;
            int rv = clt.RenewSubscription(currentLicenseId, frmRenew.dateTimePickerNew.Value);
            Reload();
        }

        private void revokeToolStripMenuItem_Click(object sender, EventArgs e)
        {
            DialogResult dr = MessageBox.Show(this, string.Format("Are you sure you want to revoke License Id {0} with Product key {1} ?\nThis action is IRREVERSIBLE",currentLicenseRow["Id"],currentLicenseRow["ProductKey"]), "Revoke Confirmation", MessageBoxButtons.YesNo);
            if (dr != DialogResult.Yes) return;
            clt.RevokeLicense(currentLicenseId);
            Reload();
        }

        private void propertiesToolStripMenuItem_Click(object sender, EventArgs e)
        {
            FormProperties frmProperties = new FormProperties(currentLicenseRow);
            frmProperties.ShowDialog(this);
        }


        static ulong DataFromInfos(byte[] Infos)
        {
            ulong data = 0;
            if ((Infos[2] & 0x80) != 0) // Multiple
            {
                data |= (ulong)(Infos[0] & 0x7FUL);
            }
            else
            {
                data |= (ulong)(Infos[0] & 0x60UL);
            }
            if (Infos[10] != 0 || Infos[11] != 0) data |= 0x80;
            data |= (ulong)Infos[3] << 8;
            data |= (ulong)Infos[8] << 16;
            data |= (ulong)Infos[9] << 24;
            ulong hash = Utilities.ReadUInt32(Infos, 12);
            hash <<= 32;
            data |= hash;
            return data;
        }

        private void newLicenseToolStripMenuItem_Click(object sender, EventArgs e)
        {
            Guid newLicenseId;
            uint newLicenseNum;
            clt.GetLicenseIds(out newLicenseNum, out newLicenseId);

            FormNewLicense frmNewLicense = new FormNewLicense(newLicenseNum, newLicenseId);
            if (frmNewLicense.ShowDialog(this) != DialogResult.OK) return;
            if (frmNewLicense.assemblyName == null) return;
            byte[] Infos = frmNewLicense.LicenseInfos;
            byte[] License = CloudLicensingClient.CreateCloudLicense(PublisherKey, newLicenseId.ToByteArray(), frmNewLicense.assemblyName, Infos, Keyset);
            ulong data = DataFromInfos(Infos);
            string newProductKey = CloudLicensingClient.EncodeProductKey(newLicenseNum, data, Keyset);
            clt.UploadLicense(newProductKey, frmNewLicense.FriendlyName, License);
            MessageBox.Show(this, newProductKey, "New License");
            Reload();
        }

        private void revokeEnabledToolStripMenuItem_Click(object sender, EventArgs e)
        {
            bool enabled = revokeEnabledToolStripMenuItem.Checked;
            revokeToolStripMenuItem.Enabled = enabled;
            revokeToolStripMenuItem1.Enabled = enabled;
        }

        private void refreshToolStripMenuItem_Click(object sender, EventArgs e)
        {
            Reload();
        }
    }
}
