﻿using System;
using System.Collections.Generic;
using System.Text;

namespace MyLicenses
{
    class Utilities
    {
        public static void Write(byte[] buffer, int index, ulong value)
        {
            for (int i = 0; i < 8; i++)
            {
                buffer[index + i] = (byte)(value & 0xFF);
                value >>= 8;
            }
        }

        public static void Write(byte[] buffer, int index, long value)
        {
            for (int i = 0; i < 8; i++)
            {
                buffer[index + i] = (byte)(value & 0xFF);
                value >>= 8;
            }
        }

        public static void Write(byte[] buffer, int index, uint value)
        {
            for (int i = 0; i < 4; i++)
            {
                buffer[index + i] = (byte)(value & 0xFF);
                value >>= 8;
            }
        }

        public static void Write(byte[] buffer, int index, int value)
        {
            for (int i = 0; i < 4; i++)
            {
                buffer[index + i] = (byte)(value & 0xFF);
                value >>= 8;
            }
        }

        public static void Write(byte[] buffer, int index, ushort value)
        {
            buffer[index] = (byte)(value & 0xff);
            value >>= 8;
            buffer[index + 1] = (byte)(value & 0xff);
        }

        public static ulong ReadUInt64(byte[] buffer, int index)
        {
            ulong value = 0;
            for (int i = 0; i < 8; i++)
            {
                value <<= 8;
                value |= buffer[index + 7 - i];
            }
            return value;
        }

        public static long ReadInt64(byte[] buffer, int index)
        {
            long value = 0;
            for (int i = 0; i < 8; i++)
            {
                value <<= 8;
                value |= buffer[index + 7 - i];
            }
            return value;
        }

        public static uint ReadUInt32(byte[] buffer, int index)
        {
            uint value = 0;
            for (int i = 0; i < 4; i++)
            {
                value <<= 8;
                value |= buffer[index + 3 - i];
            }
            return value;
        }

        public static int ReadInt32(byte[] buffer, int index)
        {
            int value = 0;
            for (int i = 0; i < 4; i++)
            {
                value <<= 8;
                value |= buffer[index + 3 - i];
            }
            return value;
        }

        public static ushort ReadUInt16(byte[] buffer, int index)
        {
            ushort value = buffer[index + 1];
            value <<= 8;
            value |= buffer[index];
            return value;
        }

        public static byte[] Extract(byte[] buffer, int index, int count)
        {
            byte[] result = new byte[count];
            Array.Copy(buffer, index, result, 0, count);
            return result;
        }

        public static string GetSzAscii(byte[] data)
        {
            if (data == null) return null;
            if (data.Length == 0) return string.Empty;
            int sz = 0;
            for (; sz < data.Length && data[sz] != 0; sz++) ;
            return System.Text.Encoding.ASCII.GetString(data, 0, sz);
        }

        public static string GetSzUtf(byte[] data)
        {
            if (data == null) return null;
            if (data.Length == 0) return string.Empty;
            int sz = 0;
            for (; sz < data.Length && data[sz] != 0; sz++) ;
            return System.Text.Encoding.UTF8.GetString(data, 0, sz);
        }

        public static string GetSzAscii(byte[] data, int index)
        {
            if (data == null) return null;
            if (data.Length == 0) return string.Empty;
            int sz = 0;
            for (int i = index; i < data.Length && data[i] != 0; i++) sz++;
            return System.Text.Encoding.ASCII.GetString(data, index, sz);
        }

        public static string GetSzUtf(byte[] data, int index)
        {
            if (data == null) return null;
            if (data.Length == 0) return string.Empty;
            int sz = 0;
            for (int i = index; sz < data.Length && data[i] != 0; i++) sz++;
            return System.Text.Encoding.UTF8.GetString(data, index, sz);
        }

        /*
        static string Perm32 = "0123456789ACDEFGHJKLMNPQRTUVWXYZ";
        static string Perm36 = "0123456789ACDEFGHJKLMNPQRTUVWXYZBIOS";
        static byte[] PkEmpty = { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 };

        public static byte[] Pk2Uid(string pk)
        {
            string npk = pk.Replace("-", "");
            if (npk.Length != 25) return PkEmpty;
            byte[] id = new byte[16];
            for (int i = 0; i < 3; i++)
            {
                ulong temp = 0;
                for (int j = 0; j < 8; j++)
                {
                    int v = Perm32.IndexOf(npk[i * 8 + j]);
                    if (v < 0) return PkEmpty;
                    temp <<= 5;
                    temp |= (byte)(v & 0xFF);
                }
                for (int j = 4; j >= 0; j--)
                {
                    id[i * 5 + j] = (byte)(temp & 0xFF);
                    temp >>= 8;
                }
            }
            int tmp = Perm36.IndexOf(npk[24]);
            if (tmp < 0) return PkEmpty;
            id[15] = (byte)(tmp & 0xFF);
            return id;
        }

        public static string Uid2Pk(byte[] id)
        {
            byte[] npk = new byte[25];
            for (int i = 0; i < 3; i++)
            {
                ulong temp = 0;
                for (int j = 0; j < 5; j++)
                {
                    temp <<= 8;
                    temp |= id[i * 5 + j];
                }
                for (int j = 7; j >= 0; j--)
                {
                    npk[i * 8 + j] = (byte)Perm32[(int)(temp & 0x1F)];
                    temp >>= 5;
                }
            }
            npk[24] = (byte)Perm36[id[15] % 36];
            string pk = "";
            for (int i = 0; i < 25; i++)
            {
                if (i > 0 && (i % 5) == 0) pk += "-";
                pk += (char)npk[i];
            }
            return pk;
        }
        */

    }
}
